<?php

/**
 * Application logic implementing user account page
 *
 */
class Account
{
	private $get;
	private $post;
	private $smarty;
	private $db;
	private $auth_module;


	/**
	 * Constructor
	 *
	 * @param array $post sanitized POST request
	 * @param array $get sanitized GET
	 */
	public function __construct($post, $get)
	{
		$this->get = $get;
		$this->post = $post;
		$this->smarty = new Smarty();
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
	}

	/**
	 * Execute page
	 *
	 */
	public function execute()
	{
		$user = $this->auth_module->get_login();
		$this->smarty->assign('level', $this->auth_module->get_level());
		$this->smarty->assign('logged_info', $user);
		if (!empty($user)) {
			$f_track = fopen("/usr/local/track/pubs.txt", "a");
			fprintf($f_track, $user);
			fprintf($f_track, "\n");
			fclose($f_track);
			
			try {
				if ($this->post['change_password'])
					$this->display_change_password();
				elseif ($this->post['change_mail'])
					$this->display_change_mail();
				elseif ($this->get['form'] == 'annotations')
					$this->display_annotations();
				elseif ($this->get['form'] == 'logout')
					$this->display_logout();
				else//if ($this->get['form'] == 'profile')
					$this->display_profile();
			} catch (SAuthException $se) {
				die($se->getMessage());
			} catch (Exception $ex) {
				$this->smarty->assign('operation_success', $ex->getMessage());
				$this->display_change_password();
			}
		} else {
			try {
				if ($this->post['login_user'])
					$this->action_login();
				elseif ($this->post['forgot_password'])
					$this->action_announce_forgotten_pass();

				elseif ($this->get['form'] == 'register')
					$this->display_register();
				elseif ($this->get['form'] == 'forgot_password')
					$this->display_forgot_password();
				elseif ($this->get['form'] == 'retrieved_password')
					$this->display_retrieved_password();
				else
					$this->display_login();
			} catch (SAuthException $se) {
				die($se->getMessage());
			}
		}

	}

	/**
	 * Displays all publications annoted by the current user
	 *
	 */
	private function display_annotations()
	{
		$data = $this->db->getAll("SELECT p.id as pid, p.title as pub_title, a.text as text, a.global_scope as scope
			FROM `publication` p JOIN `annotation` a on a.publication_id = p.id
			WHERE a.submitter_id = '".$this->auth_module->get_id()."'
			ORDER BY a.global_scope desc, a.text asc", array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data))
			throw new SAuthException($data->getMessage());
		$this->smarty->assign('annotations', $data);

		$this->smarty->assign('form', 'annotations');
		$this->smarty->display('account.tpl');
	}

	/**
	 * Tries to change current user's password and displays the result
	 *
	 */
	private function display_change_password()
	{
		try
		{
			$current_pass = trim($this->post['current_pass']);
			$pass = trim($this->post['pass']);
			if (trim($this->post['pass_repetition']) != $pass)
				throw new Exception("Password and password repetition are different.");
			$this->auth_module->set_password($pass, $current_pass);
			$this->smarty->assign('operation_success', 'The password has been successfully changed.');
		}
		catch (Exception $ex)
		{
			$this->smarty->assign('operation_error', $ex->getMessage());
		}
		$this->display_profile();
	}
	/**
	 * Tries to change current user's Email and displays the result
	 *
	 */
	private function display_change_mail()
	{
		try
		{
			$current_mail = trim($this->post['current_mail']);
			$new_mail = trim($this->post['new_mail']);

			$this->auth_module->set_mail($new_mail, $current_mail);
			$this->smarty->assign('operation_success', 'The Email has been successfully changed.');
		}
		catch (Exception $ex)
		{
			$this->smarty->assign('operation_error', $ex->getMessage());
		}
		$this->display_profile();
	}

	/**
	 * Logouts and displays the result
	 *
	 */
	private function display_logout()
	{
		$this->auth_module->logout();
		$this->smarty->assign('logged_info', null);
		$this->smarty->assign('operation_success', 'You have been logged out.');
		$this->smarty->assign('form', 'logout');
		$this->display_login();
	}

	/**
	 * Displays current user's profile
	 *
	 */
	private function display_profile()
	{
		$this->smarty->assign('form', 'profile');
		$credentials = $this->auth_module->get_credentials();
		$this->smarty->assign('role', $this->auth_module->get_role_name());
		$this->smarty->assign('login', $credentials['login']);
		$this->smarty->assign('name', $credentials['name']);
		$this->smarty->assign('surname', $credentials['surname']);
		$this->smarty->assign('email', $credentials['email']);

		$this->smarty->display('account.tpl');
	}

	/**
	 * Tries to login user and displays the result
	 *
	 */
	private function action_login()
	{
		try
		{
			$res = $this->auth_module->login($this->post['login'], $this->post['pass']);
			if ($res)
			{
				$this->smarty->assign('logged_info', $this->auth_module->get_login());
				global $app_path;
				if ($this->get['ref'])
					header('location: http://'.$app_path.$this->get['ref']);
				else
					header("location: http://$app_path/");
			}
			else
			{
				throw new Exception('Wrong password.');
			}
		}
		catch (Exception $ex)
		{
			$this->smarty->assign('operation_error', $ex->getMessage());
			$this->display_login();
		}
	}

	/**
	 * Displays login form where user should enter login and password
	 *
	 */
	private function display_login()
	{
		$this->smarty->assign('form', 'login');
		$this->smarty->display('account.tpl');
	}

	/**
	 * Display form solving forgotten passwords
	 *
	 */
	private function display_forgot_password()
	{
		$this->smarty->assign('form', 'forgot_password');
		$this->smarty->display('account.tpl');
	}

	/**
	 * Process request on solving a forgotten password
	 *
	 */
	private function action_announce_forgotten_pass()
	{
		try
		{
			$email = trim($this->post['email']);
			if (!valid_email($email))
				throw new Exception('Emailová adresa není platná');
			$this->auth_module->forgotten_pass_req($email);
			$this->smarty->assign('operation_success', 'Confirming email has been sent, check your mailbox.');
			$this->smarty->assign('form', 'login');
			$this->smarty->display('account.tpl');
		}
		catch (SAuthException $se)
		{
			die($se->getMessage());
		}
		catch (Exception $ex)
		{
			$this->smarty->assign('operation_error', $ex->getMessage());
			$this->display_forgot_password();
		}
	}

	/**
	 * Display new password
	 *
	 */
	private function display_retrieved_password()
	{
		try
		{
			$id = $this->get['submitter_id'];
			$uid = $this->get['uid'];
			$this->auth_module->reset_password($id, $uid, $login, $password);
			$this->smarty->assign('login', $login);
			$this->smarty->assign('pass', $password);
			$this->smarty->assign('form', 'retrieved_password');
		}
		catch (Exception $ex)
		{
			$this->smarty->assign('operation_error', $ex->getMessage());
		}
		$this->smarty->display('account.tpl');
	}
}


?>