<?php


/**
* Converts document text to the database
*
*/
class Convertor
{
	var $publication_id;
	var $title;
	var $fulltext;
	var $filename, $dst_filename, $txt_filename;
	var $protocol;
	var $db;

	/**
	 * Constructor
	 *
	 * @param PEAR::DB $db
	 * @param int $publication_id
	 * @param string $title optional title of the publication, if not given, the convertor will get it from the db
	 */
	function __construct(&$db, $publication_id, $title='')
	{
		$this->db = $db;
		$this->publication_id = $publication_id;
		if ($title)
			$this->title = $title;
		else {
			$title = $this->db->getOne("SELECT title FROM `publication` WHERE id = ?", $publication_id);
			if (PEAR::isError($title))
				throw new Exception($title->getMessage());
		}
	}

	/**
	 * Add text from the document to the buffer
	 *
	 * @param string $filename
	 * @return int result of conversion
	 */
	public function add_file($filename)
	{
		$this->filename = $filename;
		$narr = explode(".", $filename);
		$this->dst_filename = realpath("../storage/$this->publication_id/$filename");
		$this->txt_filename = $this->dst_filename.'.txt';
		$extension = strtolower($narr[count($narr)-1]);
		switch ($extension)
		{
			case "pdf":
				return $this->pdf();
				break;
			case "ps":
				return $this->ps();
				break;
			case "doc":
				$this->doc();
				break;
			case "txt":
				return $this->txt();
				break;
		}
	}

	/**
	 * Put the text from all added documents to the database
	 *
	 */
	public function finish()
	{
		$row = $this->db->getRow("SELECT content FROM `documents` WHERE publication_id='$this->publication_id'");
		if (PEAR::isError($row))
			throw new Exception($row->getMessage());
		if ($row) //record already exists, so update
		{
			$content = $row[0].addslashes(remove_diac($this->fulltext));
			$res = $this->db->query("UPDATE `documents` SET
				content='$content'
				WHERE publication_id='$this->publication_id'");
			if (PEAR::isError($res))
			{
				throw new Exception($res->getMessage());
			}
		}
		else //insert new record
		{
			$sth = $this->db->prepare("INSERT INTO `documents`
				(`publication_id`, `title`, `content`) VALUES (?, ?, ?)");
			if (PEAR::isError($sth))
				throw new Exception($sth->getMessage());
			$res =& $this->db->execute($sth, array($this->publication_id, $this->title, addslashes(remove_diac($this->fulltext))));
			if (PEAR::isError($res))
				throw new Exception($res->getMessage());
		}
		return $this->protocol;
	}

	/**
	 * For getting text out of the pdf documents
	 *
	 */
	private function pdf()
	{
		//get publication text
		//exec(realpath('../libs/xpdf/x86-win32/pdftotext.exe')." $this->dst_filename $this->txt_filename", $lines, $return_value);
		//exec(realpath('../libs/xpdf/x86-linux/pdftotext')." $this->dst_filename $this->txt_filename", $lines, $return_value);
		exec(realpath('../libs/xpdf/x86-freebsd/pdftotext')." $this->dst_filename $this->txt_filename", $lines, $return_value);
		switch ($return_value)
		{
			//Error opening document.
			case 1:
				throw new Exception("Error opening document $this->dst_filename\n");
			//Error opening an output file.
			case 2:
				throw new Exception("Error opening temporary file \"$this->txt_filename\"\n");
			//Error related to PDF permissions.
			case 3:
				$this->protocol .= "File \"$this->filename\" is procected against text copying.\n";
				break;
			//Other error.
			case 99:
				$this->protocol .= "During conversion of \"$this->filename\" an error occured. File is propably demaged.\n";
				break;
		}
		//save document text to database
		if (file_exists($this->txt_filename))
		{
			$fh = fopen($this->txt_filename, 'r');
			$this->fulltext .= "--- $this->filename ---\n".
						fread($fh, filesize($this->txt_filename)).
						"\n\n\n";
			fclose($fh);
			unlink($this->txt_filename);
		}
	}

	/**
	 * For getting text out of the postscript documents
	 *
	 */
	private function ps()
	{
		//get publication text
		system("ps2ascii $this->dst_filename $this->txt_filename");
		//save document text to database
		if (file_exists($this->txt_filename))
		{
			$fh = fopen($this->txt_filename, 'r');
			$this->fulltext .= "--- $this->filename ---\n".
						fread($fh, filesize($this->txt_filename)).
						"\n\n\n";
			fclose($fh);
			unlink($this->txt_filename);
		}
	}

	private function doc()
	{
		//throw new Exception("Not implemented");
	}

	/**
	 * For txt documents
	 *
	 */
	private function txt()
	{
		$fh = fopen($this->dst_filename, 'r');
		$this->fulltext .= "--- $this->filename ---\n".
					fread($fh, filesize($this->dst_filename)).
					"\n\n\n";
		fclose($fh);
		return $this->protocol;
	}
}

?>