<?php
require_once('Contents.inc.php');

/**
 * Managing authors
 *
 */
final class Contents_authors extends Contents {

	/**
	 * Execute contents tab 6
	 *
	 */
	public function execute() {
		include '../help/Help.php';
		if ($this->post['add_author']) {
			try {
				$affected = $this->add_author($this->post['name'], $this->post['middlename'], $this->post['surname']);
				if ($affected)
					$this->smarty->assign('operation_success', 'Author has been added.');
				else {
						$this->smarty->assign('operation_error', "Problem.");
					}

				$this->display_authors_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_authors_list();
			}
		} elseif ($ea=$this->post['edit_author']) {//$ea is array with only one element in format: $author_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $author_id => $v) {
					$this->display_edit_author($author_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($ea=$this->post['display_publications']) {
			try {
				if(is_array($ea)) foreach($ea as $author_id => $v) {
					$_SESSION["author_id"] = $author_id;

					$this->display_publications($author_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($ea=$this->post['overview']) {
			try {
				if(is_array($ea)) foreach($ea as $publication_id => $v) {
					$this->display_overview($publication_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		}
		elseif ($this->post['delete_files'] && $this->get['pid']){
			try {
				$this->delete_files($this->get['pid']);
				$this->smarty->assign('operation_success', 'Files has been deleted.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		} elseif ($this->post['add_file'] && $this->get['pid']) {
			try {
				$this->add_file($this->get['pid']);
				$this->smarty->assign('operation_success', 'File has been added.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		}			
 		elseif ($this->post['save_edited_author']) {
			try {
				$this->save_author($this->post['author_id'], $this->post['name'], $this->post['middlename'], $this->post['surname']);
				$this->smarty->assign('operation_success', 'Author has beed updated.');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=6');
				$this->smarty->display('contents_authors.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_author($this->post['author_id']);
			}
		} elseif ($da=$this->post['delete_author']) {//$da is array with only one element in format: [...] => $author_id
			try {
				if(is_array($da)) foreach($da as $author_id => $v) {

					$affected = $this->delete_author($author_id);
					if ($affected)
						$this->smarty->assign('operation_success', 'Autor has been deleted.');
					else {
						$this->smarty->assign('operation_error', "The record can't be deleted, it's used by some publications.");
					}
					$this->display_authors_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} else {
			$this->display_authors_list();
		}
	}

	/**
	 * Show list of authors
	 *
	 */
	private function display_authors_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY middlename"; break;
			case(3): $order_by = "ORDER BY surname"; break;
			default: $order_by = 'ORDER BY surname, name';
		}
		$sql = "SELECT *
				FROM `author` ".$order_by;
		$sql_sec = "
				SELECT author_id
				FROM `author_has_publication`
				GROUP BY author_id
				HAVING COUNT(author_id) > 0";
		$author = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($author)) foreach ($author as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$author[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_author', 1);
		$this->smarty->assign('author', $author);
		$this->smarty->display('contents_authors.tpl');
	}

	/**
	 * Display edit form of the author
	 *
	 * @param int $author_id
	 */
	private function display_edit_author($author_id) {
		$sql = "SELECT *
				FROM `author`
				WHERE id='$author_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_author', 1); //determine which part of template to use
		$this->smarty->assign('author_id', $author_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('middlename', $data['middlename']);
		$this->smarty->assign('surname', $data['surname']);
		$this->smarty->display('contents_authors.tpl');
	}

	/**
	 * Validate data sent by web form
	 *
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function validate_author($name, $middlename, $surname) {
		if (strlen($name) > 20 || !$name) throw new Exception('Name length should be 1-20 characters long.');
		if (strlen($middlename) > 20) throw new Exception('Length of middlename should be at most 20 chars.');
		if (strlen($surname) > 30 || !$surname) throw new Exception('Surname length should be 1-30 characters long.');
	}

	/**
	 * Update name and description of any author identified by id
	 *
	 * @param int $author_id
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function save_author($author_id, $name, $middlename, $surname) {
		$this->validate_author($name, $middlename, $surname);
		if ($this->db->getOne("SELECT id FROM `author` WHERE name='$name' AND middlename='$middlename' AND surname='$surname' AND id<>'$author_id'"))
			throw new Exception("The same author is already in the database.");
		$res =& $this->db->query("UPDATE `author` SET name='$name', middlename='$middlename', surname='$surname' WHERE id='$author_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete author
	 *
	 * @param int $author_id
	 */
	private function delete_author($author_id) {
		//$this->db->autoCommit(false);

		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
			$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE author_id='$author_id'");
			if(PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage);
		}*/	

		//$res =& $this->db->query("DELETE FROM `author` WHERE id='$author_id' AND '$author_id' NOT IN
		//		(SELECT DISTINCT author_id FROM `author_has_publication`) ");
		//if(PEAR::isError($res)) {
		//	$this->db->rollBack();
		//	throw new SAException($res->getMessage);
		//}
		//$this->db->commit();
		$affected = delete_author($this->db, $author_id);
		return $affected;
	}

	/**
	 * Add new author into the database
	 *
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function add_author($name, $middlename, $surname) {
		$this->validate_author($name, $middlename, $surname);
		$affected = add_author($this->db, $name,$middlename, $surname,$this->auth_module->get_id());
		return $affected;
	}

	/**
	 * Display choose form of publications
	 *
	 * 
	 */
	private function display_publications($author_id) {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY w_year"; break;
			case(3): $order_by = "ORDER BY w_from"; break;
			case(4): $order_by = "ORDER BY w_to"; break;
			case(5): $order_by = "ORDER BY location"; break;
			case(6): $order_by = "ORDER BY isbn"; break;
			case(7): $order_by = "ORDER BY description"; break;
			default: $order_by = "ORDER BY title";
		}

		$sql = "SELECT id, title, submitter_id, pub_type
				FROM `publication`
				WHERE id IN

				(SELECT publication_id from author_has_publication where author_id = $author_id)

				".$order_by; //select only complete conference2 with name and year, ommit 'private' non-complete records. Allow displaying non-complete records only to current user

		$publications = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);

		$this->smarty->assign('display_publications', 1);
		$this->smarty->assign('publications', $publications);
		$this->smarty->display('contents_authors.tpl');

	}

public function display_overview($publication_id)
	{
		$data = get_all_publication_data($this->db, $publication_id);
		$date_arr = explode('-', $data['issue_date']);
		foreach ($data as $i => $v)
		{
			$this->smarty->assign($i, $v);
		}

		//attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$publication_id'";
		$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($attributes)) {
			throw new Exception($attributes->getMessage());
		}
		$s = '';
		if (is_array($attributes))
		foreach ($attributes as $arr)
		{
			$s .=
			'<tr>
				<td><input type="checkbox" name="propagated_attributes[]" value="'.$arr['id'].'" '.($arr['confirmed']?'checked="checked" disabled="disabled"':'').' /></td>
				<td><strong>'.$arr['name'].'</strong></td>'.'
				<td>'.htmlspecialchars($arr['description']).'</td>'.'
				<td>'.htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$publication_id'")).'</td>
			</tr>';
		}
		$this->smarty->assign('attributes', $s);

		$user_name = $this->db->getOne("SELECT name FROM `submitter` WHERE id=?", $data['submitter_id']);
		$user_surname = $this->db->getOne("SELECT surname FROM `submitter` WHERE id=?", $data['submitter_id']);
		if (PEAR::isError($user_name))
			throw new SAException($user_name->getMessage());
		if (PEAR::isError($user_surname))
			throw new SAException($user_surname->getMessage());
		$this->smarty->assign('user_name', $user_name);
		$this->smarty->assign('user_surname', $user_surname);

		//issue_date
		$issue_date_a = $this->db->getOne("SELECT issue_date FROM `publication` WHERE id=$publication_id");
		$issue_date_a=substr($issue_date_a,0, 7);
		$this->smarty->assign('issue_date_a', $issue_date_a);


		$publisher=$data['publisher_id'];
		$publisher_address = $this->db->getOne("SELECT address FROM `publisher` WHERE id=?", $data['publisher_id']);
		$this->smarty->assign('publisher_address', $publisher_address);


		$conference_year_id=$data['conference_year_id'];	
		$journal_id=$data['journal_id'];	

		$authors = get_authors($this->db, $publication_id);
		$categories = get_categories($this->db, $publication_id);
		$conference_year= get_conference_year_data($this->db, $conference_year_id);
		$conference2= get_conference2_data($this->db, $conference_year_id);
		$journal= get_journal_data($this->db, $journal_id);


		$this->smarty->assign('short_look', 1); //determine which part of template to use
		$this->smarty->assign('pid', $publication_id);
		$this->smarty->assign('title', $data['title']);
		$this->smarty->assign('booktitle', $data['booktitle']);
		$this->smarty->assign('pages', $data['pages']);
		$this->smarty->assign('pub_type', $data['pub_type']);
		$this->smarty->assign('isbn', $data['isbn']);

		$dp = new DocumentProvider($this->db, true);
		$files = $dp->get_file_links($publication_id);
		$this->smarty->assign('attached_files', $files);

		$this->smarty->assign('authors', $authors);
		$this->smarty->assign('categories', $categories);
		$this->smarty->assign('conference_year', $conference_year['conference_year']);
		$this->smarty->assign('conference2', $conference2['conference2']);
		$this->smarty->assign('journal', $journal['journal']);
		$this->smarty->assign('issn', $journal['issn']);

		$this->smarty->assign('tab', 6);

		$submitter_id=$this->auth_module->get_id();
		$publ_count = $this->db->getOne("SELECT COUNT(*) FROM `submitter_has_publication` WHERE publication_id=$publication_id AND submitter_id!=$submitter_id");
		$this->smarty->assign('publ_count', $publ_count);


		$this->smarty->display('contents_authors.tpl');

	}


	/**
	 * Delete files formerly attached to the publication from its storage location
	 *
	 * @param int $pid
	 */
	public function delete_files($publication_id)
	{
		$dirpath = "../storage/$publication_id/";
		$handle = @opendir($dirpath);
		if (!$handle)
			return;
		while ($file = readdir($handle))
		{
			if (is_file($path="../storage/$publication_id/".$file))
				unlink($path);
		}
		closedir($handle);
		$res = $this->db->query("DELETE FROM `documents` WHERE publication_id=?", $publication_id);
		if (PEAR::isError($res))
			throw new SAException($res->getMessage());
    	if (!rmdir($dirpath))
    		throw new SAException("Directory couldn't be deleted.");
	}

	/**
	 * Add document to the publication
	 *
	 * @param int  $publication_id
	 * @param string $title publication title
	 */
	public function add_file($publication_id)
	{

		$file =& $_FILES['file']['tmp_name'];
		$name =& $_FILES['file']['name'];
		$extension = substr($name, strrpos($name, ".") + 1);
		if ((!is_uploaded_file($file) && $file))
			throw new SAException("Not an uploaded file!");
		if ($file) {
			$convertor = new Convertor($this->db, $publication_id);
			if(!file_exists("../storage/$publication_id"))
				mkdir("../storage/$publication_id");
			$res = move_uploaded_file($file, "../storage/$publication_id/".$name);
			$convertor->add_file($name);
			if (!$res) {
				throw new SAException("Can't move uploaded file.");
			}
			$convertor->finish();
		}
	}



}
?>