<?php



/**
 * Generates form as an interface for BibTeX import
 *
 */
class Import
{
	private $db;
	private $smarty;
	private $request;
	private $auth_module;
	private $permitted_extensions;


	/**
	 * Constructor
	 *
	 * @param array $post sanitized post data
	 * @param array $get sanitized get data
	 * @param int $permitted_extensions  permitted file extensions
	 */
	public function __construct($request, $permitted_extensions)
	{
		$this->permitted_extensions = $permitted_extensions;
		$this->request = $request;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);

		$this->smarty = new Smarty;
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
		//if no user logged in, force authorization
		if (!is_numeric($this->auth_module->get_id()))
		{
			$this->smarty->assign('refresh', "1 ; URL=$account_url"."index.php?ref=/import/");
			$this->smarty->assign('operation_error', 'To the import interface you must <a href="'.$account_url.'index.php?ref=/load/">log in</a>.');
			$this->smarty->display('import_login.tpl');
			die();
		}
		if ($this->auth_module->get_level() > 1)
		{
			throw new SIException("You don't have rights to import publication.");
		}
	}

	/**
	 * Execute page, entry point of Import class
	 *
	 */
	public function execute()
	{
		include '../help/Help.php';
		unset($_SESSION['feedback_st2']);

		$request = $this->request;
		$this->smarty->assign('level', $this->auth_module->get_level());
		if ($request['submit_import'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				if (!$request['bibtex'])
					throw new Exception('Bibtex definiton is missing');
				$this->display_st2();
			} catch (SIException $ex) {
				die($ex->getMessage());
			} catch (Exception $ex) {
				$this->smarty->assign('operation_error', $ex->getMessage());
				$this->display_st1();
			}
		}
		elseif ($request['submit_finish'])
		{
			try {
				$this->validate_files();
				$this->finish();
			} catch (SIException $ex) {
				die($ex->getMessage());
			} catch (Exception $ex) {
				$this->smarty->assign('operation_error', $ex->getMessage());
				$this->display_st2();
			}
		}
		elseif ($request['add_conference2'])
		{
			try {
				extract($request);
				$conference2_id = add_conference2($this->db, $conference_name2, $conference_abbreviation2, $this->auth_module->get_id(), $conference_description2, $conference_first_year2);
				$this->smarty->assign('operation_success', 'Conference2 has been added.');

				unset ($_SESSION['new_conference2']);
				$_SESSION['new_conference2'] = $conference2_id;

				$this->display_st2_2();

			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		//add conference_year with publisher
		elseif ($request['add_conference_year_c']) {
			try {
				extract($request);
					if(!$publisher_c){
						$publisher_c=0;
					}
				
				$conference_year_id = add_conference_year_c($this->db, $conference_year_name, $conference2, $conference_year_year, $conference_year_from, $conference_year_to, $conference_year_location, $conference_year_isbn, $conference_year_description, $publisher_c, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference year has been added.');

				unset ($_SESSION['new_conference_year']);
				$_SESSION['new_conference_year'] = $conference_year_id;

				$this->display_st2_2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}

		}
		elseif ($request['add_journal'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				add_journal($this->db, $request['journal_name'], $request['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been added.');
				$this->display_st2_2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		elseif ($request['add_publisher'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				add_publisher($this->db, $request['publisher_name'], $request['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been added.');
				$this->display_st2_2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['add_author']) {
			try
			{
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$a=add_author($this->db, $request['author_name'], $request['author_middle_name'], $request['author_surname'], $this->auth_module->get_id());

				$a=array(0 => $a);

				if (is_array(($a)))
			 	{
				 	foreach ($a as $i => $value)
				 	{
				 		if (sizeof($_SESSION["selected_authors"]) == 0){
				 			$_SESSION["selected_authors"] = array($value);
				 		}else{
				 			$_SESSION["selected_authors"] = array_merge($_SESSION["selected_authors"], array($value));
				 		}
					}
				 	$_SESSION["selected_authors"] = array_unique($_SESSION["selected_authors"]);
			 	}


				$this->smarty->assign("operation_success", "Author has been added.");
				$this->display_st2_2();

			}
			catch (SLException $s)
			{
				die($s->getMessage());
			}
			catch (Exception $e)
			{
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['select_author']) //choose author, copy him to the right listbox
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['authors'];
			 	if (is_array(($a)))
			 	{
				 	foreach ($a as $i => $value)
				 	{
				 		if (sizeof($_SESSION["selected_authors"]) == 0){
				 			$_SESSION["selected_authors"] = array($value);
				 		}else{
				 			$_SESSION["selected_authors"] = array_merge($_SESSION["selected_authors"], array($value));
				 		}
					}
				 	$_SESSION["selected_authors"] = array_unique($_SESSION["selected_authors"]);
			 	}
			 	$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['deselect_author']) //remove author from selection
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['selected_authors'];
			 	if (is_array($a))
			 	{
				 	foreach ($a as $value)
				 	{
				 		foreach ($_SESSION["selected_authors"] as $i => $v)
				 			if ($value == $v)
				 				$_SESSION["selected_authors"][$i] = null;
				 	}
				 	$_SESSION["selected_authors"] = $this->array_shrink($_SESSION["selected_authors"]);
			 	}
			 	$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['up']) //move author up
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['selected_authors'];
			 	if (is_array($a))
			 	{
					$id = $a[0];
			 		$pos = array_search($id, $_SESSION["selected_authors"]);
			 		if ($pos != 0)
			 		{
			 			$tmp = $_SESSION["selected_authors"][$pos - 1];
			 			$_SESSION["selected_authors"][$pos - 1] = $id;
			 			$_SESSION["selected_authors"][$pos] = $tmp;
			 		}
			 	}
			 	$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['down']) //move author down
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['selected_authors'];
			 	if (is_array($a))
			 	{
					$id = $a[0];
			 		$pos = array_search($id, $_SESSION["selected_authors"]);
			 		if ($pos != count($_SESSION["selected_authors"]) - 1)
			 		{
			 			$tmp = $_SESSION["selected_authors"][$pos + 1];
			 			$_SESSION["selected_authors"][$pos + 1] = $id;
			 			$_SESSION["selected_authors"][$pos] = $tmp;
			 		}
			 	}
			 	$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}

		elseif ($request['update_conference_year_c'])
		{
			try {
				extract($request);
				if($publisher_sel_c){
					$publisher_c=$publisher_sel_c;
				}
				update_conference_year_p($this->db, $conference_year, $conference_year_name, $conference_year_year, $conference_year_from, $conference_year_to, $conference_year_location, $conference_year_isbn,$conference_year_description, $publisher_c, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference year has been updated');

				unset ($_SESSION['update_conference_year']);
				$_SESSION['update_conference_year'] = $conference_year;

				$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}

		elseif ($request['update_conference2'])
		{
			try {
				extract($request);
				update_conference2($this->db, $conference2, $conference_name2, $conference_abbreviation2, $conference_first_year2, $conference_description2, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference 2 has been updated');

				unset ($_SESSION['update_conference2']);
				$_SESSION['update_conference2'] = $conference2;

				$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}

		else if ($request['update_journal'])
		{
			try {
				update_journal($this->db, $this->request['journal'], $this->request['journal_name'], $this->request['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been updated');
				$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['update_publisher'])
		{
			try {
				update_publisher($this->db, $this->request['publisher'], $this->request['publisher_name'], $this->request['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been updated.');
				$this->display_st2_2();
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		elseif ($request['add_category'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				add_tmp_category($this->db, $request['category_name']);
				$this->smarty->assign('operation_success', 'Category will be added after finish this publication');
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['delete_added_categories'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request;
				unset($_SESSION['tmp_cat']);
				$this->smarty->assign("operation_success", "Added categories were deleted");
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else
		{
			$this->feedback_st1();
			$this->display_st1();
		}
	}

	/**
	 * Display the import form
	 *
	 */
	private function display_st1()
	{
		//categories
		$this->feedback_st1();
		$sql = "SELECT id, name
				FROM `categories`
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//added categories
		$s=null;
		if (is_array($_SESSION['tmp_cat'])) foreach($_SESSION['tmp_cat'] as $name)
		{
			$s[] = $name;
		}
		$this->smarty->assign('added_categories', stripslashes_rec(htmlspecialchars_rec($s)));

		$this->smarty->assign('stage', 0);
		$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Try to parse given BibTeX definition, save the result
	 * into $_SESSION['bibtex_parse'] and display the result.
	 *
	 */
	private function display_st2()
	{
		if (isset($_SESSION['feedback_st2'])) $this->feedback_st2(); //feedback

		
		$submitter_id = $this->auth_module->get_id();
		//parsing
		$parser = new Parser($this->request['bibtex']);
		$pub_type = $fields = $authors = null;
		$parser->parse($pub_type, $fields, $authors);
		//save the result
		$_SESSION['bibtex_parse'] = array('pub_type' => $pub_type, 'fields' => $fields, 'authors' => $authors);
		//bibtex validation
		$bibtex = Bibtex::create($pub_type);
		$report = $bibtex->validate($fields);
		$this->smarty->assign('report', $report);
		//bibtex fields
		$this->smarty->assign('pub_type', $pub_type);
		foreach ($fields as $i => $v)
			$this->smarty->assign($i, $v);
		//checked categories
		$s = null;
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories))
			foreach ($propagated_categories as $i => $v) {
				$s[] = $this->db->getOne("SELECT name FROM `categories` WHERE id='$v'");
		}
		$this->smarty->assign('categories', stripslashes_rec(htmlspecialchars_rec($s)));
		//added categories
		$s = null; $tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s[] = $v;
			}
		}
		$this->smarty->assign('tmp_cat', stripslashes_rec(htmlspecialchars_rec($s)));
		$this->smarty->assign('stage', 1);

		//check for duplicates
		$title = $fields['title'];
		if (publiation_exists($this->db, $title))
			$this->smarty->assign('warning', "Publication '$title' already exists in the database, nevertheless you can continue.");

		//select either complete publisher
		$sql = "SELECT id, name
			FROM `publisher`
			ORDER BY name";
		$this->smarty->assign('publisher_c', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//select either complete conferences2
		$sql = "SELECT id, CONCAT(abbreviation, ' (', name, ')')
			FROM `conference2`
			ORDER BY abbreviation";
		$this->smarty->assign('conference2', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//select either complete journal
		$sql = "SELECT id, CONCAT(name)
			FROM `journal`
			ORDER BY name";
		$this->smarty->assign('journals', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//select either complete publisher
		$sql = "SELECT id, CONCAT(name)
			FROM `publisher`
			ORDER BY name";
		$this->smarty->assign('publishers', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//list of authors
		$sql = "SELECT id, CONCAT_WS(' ',surname,middlename,name) AS author
				FROM `author`
				ORDER BY surname, name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SLException($arr->getMessage());
		$this->smarty->assign('authors', htmlspecialchars_rec($arr));


		/**
		 * display authors from bibtex import (if are in database)
		 *
		 */
		$authors = $_SESSION['bibtex_parse']['authors'];
		if (count($authors) != 0)
		{
			//echo(" jsou ");
			foreach ($authors as $a)
			{
			   $b = author_exists($this->db, $a['name'], $a['middlename'], $a['surname']);
			   if($b){
				$b=array(0 => $b);
				if (is_array(($b)))
				{
					foreach ($b as $i => $value)
					{
						if (sizeof($_SESSION["selected_authors"]) == 0){
							$_SESSION["selected_authors"] = array($value);
							//echo(" val "); echo($value);
		 				}
						if (sizeof($_SESSION["selected_authors"]) != 0){
							$_SESSION["selected_authors"] = array_merge($_SESSION["selected_authors"], array($value));
							//echo(" val2 "); echo($value);
						}
					}
		 			$_SESSION["selected_authors"] = array_unique($_SESSION["selected_authors"]);
 				}
			   }
			}
		}
		$author_names = array();
		try
		{
			if (sizeof($_SESSION["selected_authors"]) != 0){
				foreach ($_SESSION["selected_authors"] as $id)
				{
					$author_name = $this->db->getOne("select CONCAT_WS(' ',name,middlename,surname) AS author from author where id=?", $id);
					if (PEAR::isError($author_name))
		 	   			throw new Exception($author_name->getMessage(), 1);
		 			$author_names = array_merge($author_names, array($author_name));
				}
			}
		}
		catch (Exception $e)
		{
			throw new SLException($e->getMessage());
		}
		$this->smarty->assign("selected_authors", $_SESSION["selected_authors"]);
		$this->smarty->assign("selected_authors_names", $author_names);


		

		$this->display_st2_2();
		//$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Try to parse given BibTeX definition, save the result
	 * into $_SESSION['bibtex_parse'] and display the result.
	 *
	 * Data for add_conference2, update_conference2, add_conference_year, update_conference_year
	 * from $_SESSION['new_conference2'], $_SESSION['update_conference2'], $_SESSION['new_conference_year'], $_SESSION['update_conference_year']
	 *
	 */

	private function display_st2_2()
	{
		if (isset($_SESSION['feedback_st2'])) $this->feedback_st2(); //feedback


		$pub_type = $_SESSION['bibtex_parse']['pub_type'];
		$fields =  $_SESSION['bibtex_parse']['fields'];
		$authors = $_SESSION['bibtex_parse']['authors'];
		extract($fields);
		$this->smarty->assign('pub_type', $pub_type);
		$this->smarty->assign('title', $title);
		$this->smarty->assign('booktitle', $booktitle);
		$this->smarty->assign('author', $author);
		$this->smarty->assign('volume', $volume);
		$this->smarty->assign('number', $number);
		$this->smarty->assign('chapter', $chapter);
		$this->smarty->assign('pages', $pages);
		$this->smarty->assign('series', $series);
		$this->smarty->assign('edition', $edition);
		$this->smarty->assign('howpublished', $howpublished);
		$this->smarty->assign('type', $type);
		$this->smarty->assign('institution', $institution);
		$this->smarty->assign('school', $school);
		$this->smarty->assign('organization', $organization);
		$this->smarty->assign('year', $year);
		$this->smarty->assign('month', $month);
		$this->smarty->assign('location', $location);
		$this->smarty->assign('journal', $journal);
		$this->smarty->assign('publisher', $publisher);
		$this->smarty->assign('address', $address);
		$this->smarty->assign('isbn', $isbn);
		$this->smarty->assign('issn', $issn);
		$this->smarty->assign('note', $note);
		$this->smarty->assign('url', $url);
		$this->smarty->assign('abstract', $abstract);
		$this->smarty->assign('categories', $categories);

		//list of publisher_c
			$sql = "SELECT id, name
				FROM `publisher`
				ORDER BY name";
			$this->smarty->assign('publisher_c', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//list of conferences2
			$sql = "SELECT id, CONCAT(abbreviation, ' (', name, ')')
				FROM `conference2`
				ORDER BY abbreviation";
			$this->smarty->assign('conference2', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//list of journal
			$sql = "SELECT id, CONCAT(name)
				FROM `journal`
				ORDER BY name";
			$this->smarty->assign('journals', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//list of publisher
			$sql = "SELECT id, CONCAT(name)
				FROM `publisher`
				ORDER BY name";
			$this->smarty->assign('publishers', htmlspecialchars_rec($this->db->getAssoc($sql)));

		//list of authors
			$sql = "SELECT id, CONCAT_WS(' ',surname,middlename,name) AS author
					FROM `author`
					ORDER BY surname, name";
			$arr = $this->db->getAssoc($sql);
			if (PEAR::isError($arr))
				throw new SLException($arr->getMessage());
			$this->smarty->assign('authors', htmlspecialchars_rec($arr));

		
		// If the conference_year is detected, the appropriate fields are filled

			if(!$_SESSION['new_conference2'] && !$_SESSION['update_conference2']){
				$isbn ="";
				$conference ="";
				$conference_year_id="";
				$conference2_id="";
				$isbn = $fields['isbn'];
				$conference = $fields['booktitle'];
				//echo($conference);
				if($conference){
					$conference_year_id=$this->db->getOne("SELECT id FROM `conference_year` WHERE name='$conference'");
				}
				if($isbn){
					$conference_year_id2=$this->db->getOne("SELECT id FROM `conference_year` WHERE isbn='$isbn'");
					if($conference_year_id2){
						$conference_year_id=$conference_year_id2;
					}
				}
				if($conference_year_id){
				   unset($_SESSION['conference2_year']);
				   $_SESSION['conference2_year'] = $conference2_year_id;
				   //echo(" id "); echo($conference_year_id);
					$conference2_id=$this->db->getOne("SELECT conference2_id FROM `conference_year` WHERE id=$conference_year_id");
					//echo(" id2 "); echo($conference2_id);

					//select conferences_year
					$sql = "SELECT id, CONCAT(w_year, ' (', name, ')')
						FROM `conference_year`
						WHERE conference2_id=$conference2_id
						ORDER BY w_year";
					$this->smarty->assign('conference_year', htmlspecialchars_rec($this->db->getAssoc($sql)));

					$sql = "SELECT * FROM `conference_year` WHERE id='$conference_year_id'";
					$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
					if (PEAR::isError($data)) {
    						throw new SAException($data->getMessage());
					}

					$this->smarty->assign('conference_year_name', $data['name']);
					$this->smarty->assign('conference_year_year', $data['w_year']);
					$this->smarty->assign('conference_year_from', $data['w_from']);
					$this->smarty->assign('conference_year_to', $data['w_to']);
					$this->smarty->assign('conference_year_location', $data['location']);
					$this->smarty->assign('conference_year_isbn', $data['isbn']);
					$this->smarty->assign('conference_year_description', $data['description']);
					$this->smarty->assign('selected_publisher_c', $data['publisher_id']);

					$sql = "SELECT * FROM `conference2` WHERE id='$conference2_id'";
					$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
					if (PEAR::isError($data)) {
    						throw new SAException($data->getMessage());
					}
				
					$this->smarty->assign('conference_name2', $data['name']);
					$this->smarty->assign('conference_abbreviation2', $data['abbreviation']);
					$this->smarty->assign('conference_first_year', $data['first_year']);
					$this->smarty->assign('conference_description2', $data['description']);


					$this->smarty->assign('selected_conference2', $conference2_id);
					$this->smarty->assign('selected_conference_year', $conference_year_id);
				   
				}

			}




		// fill fields of conference2, selected_conference2 
			if($_SESSION['new_conference2'] || $_SESSION['update_conference2']){
				if($_SESSION['new_conference2']){
					$conference2_id = $_SESSION['new_conference2'];
				}
				if($_SESSION['update_conference2']){
					$conference2_id = $_SESSION['update_conference2'];
					$sql = "SELECT id, CONCAT(w_year, ' (', name, ')')
						FROM `conference_year` WHERE conference2_id= $conference2_id
						ORDER BY w_year";
					$this->smarty->assign('conference_year', htmlspecialchars_rec($this->db->getAssoc($sql)));
				}
				$this->smarty->assign('selected_conference2', $conference2_id);
				$sql = "SELECT *
					FROM `conference2`
					WHERE id='$conference2_id'";
				$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
				if (PEAR::isError($data)) {
    					throw new SAException($data->getMessage());
				}
				$this->smarty->assign('conference_name2', $data['name']);
				$this->smarty->assign('conference_abbreviation2', $data['abbreviation']);
				$this->smarty->assign('conference_first_year2', $data['first_year']);
				$this->smarty->assign('conference_description2', $data['description']);
				unset ($_SESSION['new_conference2']);
				unset ($_SESSION['update_conference2']);
			}

		// fill fields of conference_year and conference2, selected_conference2, selected_conference_year 
			if($_SESSION['new_conference_year'] || $_SESSION['update_conference_year']){
				if($_SESSION['new_conference_year']){
					$conference_year_id = $_SESSION['new_conference_year'];
				}
				if($_SESSION['update_conference_year']){
					$conference_year_id = $_SESSION['update_conference_year'];
				}

				$sql = "SELECT conference2_id
					FROM `conference_year`
					WHERE id='$conference_year_id'";
				$conference2_id = htmlspecialchars_rec($this->db->getOne($sql));

				$sql = "SELECT id, CONCAT(w_year, ' (', name, ')')
					FROM `conference_year` WHERE conference2_id= $conference2_id
					ORDER BY w_year";
				$this->smarty->assign('conference_year', htmlspecialchars_rec($this->db->getAssoc($sql)));
				$this->smarty->assign('selected_conference_year', $conference_year_id);
				$this->smarty->assign('selected_conference2', $conference2_id);
			
				$sql = "SELECT *
					FROM `conference_year`
					WHERE id='$conference_year_id'";
				$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
				if (PEAR::isError($data)) {
    					throw new SAException($data->getMessage());
				}
				$this->smarty->assign('conference_year_name', $data['name']);
				$this->smarty->assign('conference_year_year', $data['w_year']);
				$this->smarty->assign('conference_year_from', $data['w_from']);
				$this->smarty->assign('conference_year_to', $data['w_to']);
				$this->smarty->assign('conference_year_location', $data['location']);
				$this->smarty->assign('conference_year_isbn', $data['isbn']);
				$this->smarty->assign('conference_year_description', $data['description']);
				$this->smarty->assign('selected_publisher_c', $data['publisher_id']);

				$sql = "SELECT *
					FROM `conference2`
					WHERE id='$conference2_id'";
				$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
				if (PEAR::isError($data)) {
    					throw new SAException($data->getMessage());
				}
				$this->smarty->assign('conference_name2', $data['name']);
				$this->smarty->assign('conference_abbreviation2', $data['abbreviation']);
				$this->smarty->assign('conference_first_year2', $data['first_year']);
				$this->smarty->assign('conference_description2', $data['description']);
				unset ($_SESSION['new_conference_year']);
				unset ($_SESSION['update_conference_year']);
			}
	

		//If the journal is detected, the appropriate fields are filled
		if ($pub_type == 'article'){
			$issn ="";
			$journal="";
			$journal_id="";
			//echo(" tady ");echo($issn);
			$issn = $fields['issn'];
			$journal= $fields['journal'];
			//echo(" tady2 ");echo($issn);
			if($journal){
				$journal_id=$this->db->getOne("SELECT id FROM `journal` WHERE name='$journal'");
			}
			if($issn){
				$journal_id2=$this->db->getOne("SELECT id FROM `journal` WHERE issn='$issn'");
				if($journal_id2){
					$journal_id=$journal_id2;
				}
			}
			if($journal_id){
				//echo(" zname issn ");
			
				//echo(" id "); echo($journal_id);
				$this->smarty->assign('journal_name', $this->db->getOne("SELECT name FROM `journal` WHERE id=$journal_id"));
				$this->smarty->assign('journal_issn', $this->db->getOne("SELECT issn FROM `journal` WHERE id=$journal_id"));
				$this->smarty->assign('selected_journal', $journal_id);
			}
			if(!$journal_id){
				//echo(" nezname issn ");
				$this->smarty->assign('selected_journal', "");
			}
		}
		if ($pub_type != 'article'){
			$issn ="";
			//echo(" nezname issn "); echo($issn); 


		}


		//If the publisher is detected, the appropriate fields are filled
			$publisher = $fields['publisher'];
			$publisher_id=$this->db->getOne("SELECT id FROM `publisher` WHERE name='$publisher'");

			if($publisher_id){
				//echo(" znamy publisher ");
				//echo(" id "); echo($publisher_id);
				$this->smarty->assign('publisher_name', $this->db->getOne("SELECT name FROM `publisher` WHERE id=$publisher_id"));
				$this->smarty->assign('publisher_address', $this->db->getOne("SELECT address FROM `publisher` WHERE id=$publisher_id"));
				$this->smarty->assign('selected_publisher', $publisher_id);
			}
	
		$this->smarty->assign('stage', 1);
		$this->smarty->display('import_bibtex.tpl');

	}

	/**
	 * Finish importing publication, display message about success
	 *
	 */
	private function finish()
	{
		$submitter_id = $this->auth_module->get_id();
		//sanitize input
		$pub_type = addslashes_rec($_SESSION['bibtex_parse']['pub_type']);
		$fields =  addslashes_rec($_SESSION['bibtex_parse']['fields']);
		$authors = addslashes_rec($_SESSION['bibtex_parse']['authors']);

		extract($fields);

		$this->db->autoCommit(false);

		//redistribute conference, journal, publisher and author data into the database
		if ($pub_type == 'inproceedings' || $pub_type == 'proceedings')
		{
			if ($pub_type == 'inproceedings')
			{
				$c_name = $booktitle;
				$booktitle = '';
			}
			else
			$c_name = $title;
			$c_location = $location;
			$location = '';
			$c_year = $year;
			$year = '';
			$c_isbn = $isbn;

			if($publisher){
				$sql = "SELECT id FROM `publisher` WHERE name='$publisher'";
				$publisher_id=$this->db->getOne($sql);
			}
			if(!$publisher){
				$publisher_id='';
			}

			if($_POST["conference2"] && $_POST["conference_year"]){
				$conference2_id= $_POST["conference2"];
				$conference_year_id = $_POST["conference_year"];

				//change publisher name
				$sql = "SELECT publisher_id FROM `conference_year` WHERE id=$conference_year_id";
				$publisher_id=$this->db->getOne($sql);
				$sql = "SELECT name FROM `publisher` WHERE id=$publisher_id";
				$publisher=$this->db->getOne($sql);
					
				$sql = "SELECT isbn FROM `conference_year` WHERE id=$conference_year_id";
				$isbn=$this->db->getOne($sql);
			}
		}
		elseif ($pub_type == 'article')
		{
			if($_POST["journal"]){
				$journal_id = $_POST["journal"];
			}
		}

		if($_POST["publisher"]){
			$publisher_id = $_POST["publisher"];
		}

		
		if($_SESSION["selected_authors"]){
			$a = $_SESSION["selected_authors"];
			 	if (is_array(($a)))
			 	{
				 	foreach ($a as $i => $value)
				 	{
				 		if (sizeof($_SESSION["selected_authors"]) != 0){
				 			$author_ids[]= $value;
				 		}
					}
			 	}
		}


		if ($month && !is_numeric($month))
			$issue_date = $year.'-'.strmonth2nummonth($month).'-00';
		else
			$issue_date = $year.'-'.((is_numeric($month) && strlen($month==2))?$month:'00').'-00';


		//insert into `publication`
		$sql = "INSERT INTO `publication`
				(id, submitter_id, publisher_id, conference_year_id, journal_id, pub_type, title, booktitle, abstract, issue_date, isbn, volume, number, pages, note, chapter, edition, editor, howpublished, institution, organization, school, address, type_of_report, url, confirmed) VALUES
				('', '$submitter_id', '$publisher_id', '$conference_year_id', '$journal_id', '$pub_type', '$title', '$booktitle', '$abstract', '$issue_date', '$isbn', '$volume', '$number', '$pages', '$note', '$chapter', '$edition', '$editor', '$howpublished', '$institution', '$organization', '$school', '$address', '$type', '$url', 0)";
		$res = $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SLException($res->getMessage());
		}
		$publication_id = last_insert_id($this->db);

		//insert into `author_has_publication`
		if($_SESSION["selected_authors"]){
			if (sizeof($author_ids != 0))
			{
				$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?, ?)");
				if (PEAR::isError($sth))
					throw new SIException($sth->getMessage());
				$priority = 0;
				foreach ($author_ids as $_id)
				{
					$res =& $this->db->execute($sth, array($_id, $publication_id, $priority++));
					if (PEAR::isError($res))
						throw new SIException($res->getMessage());
				}
			}
		}

		//insert into submitter_has_publication
		if(isset($_POST['want_set']) && $_POST['want_set'] == 'yes'){
			//insert into `submitter_has_publication`
			$res =& $this->db->query("INSERT IGNORE INTO submitter_has_publication(submitter_id, publication_id) values($submitter_id, $publication_id)");
			if(DB::isError($res)) {
				$this->db->rollback();
				throw new SAException($res->getMessage());
			}
		}

		//insert into `categories`
		$inserted_cat_ids = null;
		$tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			$sth =& $this->db->prepare("INSERT INTO `categories` VALUES('', '$submitter_id', ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SIException($sth->getMessage());
			}
			foreach ($tmp as $name) {
				$res =& $this->db->execute($sth, stripslashes($name));//stripslashes because prepare-execute add slashes redundantly
				$inserted_cat_ids[] = last_insert_id($this->db);
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}

		//insert into `categories_has_publication`
		$tmp = $_SESSION['tmp_cat'];
		$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SIException($sth->getMessage());
		}
		if (is_array($inserted_cat_ids)) {
			foreach ($inserted_cat_ids as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories)) {
			foreach ($propagated_categories as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SIException($res->getMessage());
				}
			}
		}

		//store pdf/ppt into the filesystem
		save_documents($this->db, $publication_id, $title);

		$this->db->commit();

		unset($_SESSION['feedback_st1']);
		unset($_SESSION['bibtex_parse']);

		$this->smarty->assign('operation_success', "Publication has been successfully imported. Now you can modify the publication.");
		$this->smarty->assign('refresh', "1 ; URL=../index.php?pid=$publication_id&amp;tab=2");
		//$this->smarty->display('modify_publication.tpl');
		$this->smarty->display('import_bibtex.tpl');
	}

	/**
	 * Gives user data back to form stage 1
	 *
	 */
	private function feedback_st1()
	{
		$feedback = $_SESSION['feedback_st1'];
		$this->smarty->assign('bibtex', $feedback['bibtex']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
	}

	/**
	 * Validate - check attached files
	 *
	 */
	private function validate_files()
	{
		$permitted_extensions = $this->permitted_extensions;
		foreach($_FILES['documents']["error"] as $key => $error)
		{
			if ($error == UPLOAD_ERR_OK)
			{
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$extension = substr($name, strrpos($name, ".") + 1);
				if ($permitted_extensions)
				{
					if (!in_array($extension, $permitted_extensions))
						throw new Exception("You are adding file with a non-permited extension.");
				}
				if ((!is_uploaded_file($file) && $file))
					throw new SLException('Not an uploaded file.');
			}
		}
	}

	/**
	 * Store given documents
	 *
	 * @param int $publication_id
	 * @param string $title title of the publication
	 * @return string info about document conversion
	 */
	private function save_documents($publication_id, $title)
	{
		$sth = $this->db->prepare("INSERT INTO `documents`
		(`publication_id`, `title`, `content`) VALUES (?, ?, ?)");
		if (PEAR::isError($sth))
			throw new SLException($sth->getMessage());
		$convertor = new Convertor($publication_id, $title, $this->db);
		foreach($_FILES['documents']['error'] as $key => $error)
		{
			if ($error == UPLOAD_ERR_OK)
			{
				if(!file_exists("../storage/$publication_id"))
					mkdir("../storage/$publication_id");
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$dst_filename = realpath("../storage/$publication_id/$name");
				//save document file to filesystem
				$success = move_uploaded_file($file, $dst_filename);
				if (!$success)
					throw new SLException("Can't move uploaded file");
				$convertor->add_file($name);
			}
		}
		return $convertor->finish(); //return info about document conversion
	}

	/**
	 * Gives written data back to form stage 2
	 *
	 */
    private function feedback_st2()
    {
		$feedback = $_SESSION['feedback_st2'];
		$feedback = htmlspecialchars_rec($feedback);
		foreach ($feedback as $i => $v)
		{
			if ($i != 'authors' && $i != 'journal' && $i != 'conference' && $i != 'conference2' && $i != 'conference_year' && $i != 'publisher')
				$this->smarty->assign($i, $v);
		}

		//authors - select options
		$author_names = array();
		try
		{
			if (sizeof($_SESSION["selected_authors"]) != 0){
				foreach ($_SESSION["selected_authors"] as $id)
				{
					$author_name = $this->db->getOne("select CONCAT_WS(' ',name,middlename,surname) AS author from author where id=?", $id);
					if (PEAR::isError($author_name))
		 	   			throw new Exception($author_name->getMessage(), 1);
		 			$author_names = array_merge($author_names, array($author_name));
				}
			}
		}
		catch (Exception $e)
		{
			throw new SLException($e->getMessage());
		}
		$this->smarty->assign("selected_authors", $_SESSION["selected_authors"]);
		$this->smarty->assign("selected_authors_names", $author_names);
		$conference2_id =$feedback['conference2'];


		if (isset($_SESSION['new_conference'])){
			$conference2_id=$_SESSION['new_conference'];
			$sql = "SELECT name
				FROM `conference2`
				WHERE id=$conference2_id";
			$this->smarty->assign('conference_year_name', htmlspecialchars_rec($this->db->getOne($sql)));

			$sql = "SELECT description
				FROM `conference2`
				WHERE id=$conference2_id";
			$this->smarty->assign('conference_year_description', htmlspecialchars_rec($this->db->getOne($sql)));
		}

		if($conference2_id){
			$sql = "SELECT id, CONCAT(w_year, '(', name,')')
				FROM `conference_year` 
				WHERE `conference2_id`= $conference2_id
				ORDER BY w_year";
			$arr = $this->db->getAssoc($sql);
			if (PEAR::isError($arr))
				throw new SLException($arr->getMessage());
			$this->smarty->assign('conference_year', $arr);
		}


		//other select options
		$this->smarty->assign('selected_journal', $feedback['journal']);
		$this->smarty->assign('selected_conference', $feedback['conference']);
		$this->smarty->assign('selected_conference2', $feedback['conference2']);
		//$this->smarty->assign('selected_conference_year', $feedback['conference_year']);
		$this->smarty->assign('selected_publisher', $feedback['publisher']);

		if($_SESSION['new_conference']){
			$this->smarty->assign('selected_conference2', $_SESSION['new_conference']);
		}


	}

	/**
	 * Removes null values from an array and shriks it's content
	 *
	 * @param array $arr
	 */
	private function array_shrink($arr) {
		foreach ($arr as $i => $v)
		{
			if ($v != null)
				$ret[$i] = $v;
		}
		return $ret;
	}


	/**
	 * Delete author from temporary list
	 *
	 * @param int $author_id id of author
	 */
	private function delete_author_from_temp($author_id)
	{
		$_SESSION['selected_authors'] = array_diff($_SESSION['selected_authors'], array($author_id));
	}



}
?>